Module err

Module err 

Error module for tor-dirserver

TODO DIRMIRROR: The way on how we structure errors needs further discussion.

Enums

BuilderError
An error while building a builder struct to the target structure.