Error module for tor-dirserver
TODO DIRMIRROR: The way on how we structure errors needs further discussion.
Enums
- Builder
Error - An error while building a builder struct to the target structure.
Error module for tor-dirserver
TODO DIRMIRROR: The way on how we structure errors needs further discussion.